unit Unit5;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs,
  VclTee.TeeGDIPlus, Vcl.StdCtrls, VclTee.TeEngine, Vcl.ExtCtrls, VclTee.TeeProcs, VclTee.Chart, VclTee.TeeLisB, Vcl.Buttons, VclTee.TeeEdiGrad,
  Vcl.Imaging.jpeg, VclTee.Series, VclTee.TeeTools;

type
  TForm1 = class(TForm)
    ButtonExit: TButton;
    RadioGroup1: TRadioGroup;
    Chart1: TChart;
    ChartTool1: TCursorTool;
    procedure FormCreate(Sender: TObject);
    procedure Chart1AfterDraw(Sender: TObject);
    procedure ButtonExitClick(Sender: TObject);
    procedure RadioGroup1Click(Sender: TObject);
    procedure ChartTool1Change(Sender: TCursorTool; x, y: Integer; const XValue, YValue: Double; Series: TChartSeries; ValueIndex: Integer);
  private
    { Private declarations }
    tmpXVal, tmpYVal: Double;
    function InterpolateLineSeriesVertical(Series: TChartSeries; XValue: Double): Double; overload;
    function InterpolateLineSeriesVertical(Series: TChartSeries; FirstIndex, LastIndex: Integer; XValue: Double): Double; overload;
    function InterpolateLineSeriesHorizontal(Series: TChartSeries; YValue: Double): Double; overload;
    function InterpolateLineSeriesHorizontal(Series: TChartSeries; FirstIndex, LastIndex: Integer; YValue: Double): Double; overload;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
var
  i: Integer;
begin
  with Chart1 do
    begin
      View3D := false;
      View3DWalls := false;
      Chart3DPercent := 0;
      Legend.Visible := false;
      Title.Visible := false;
    end;

  if Chart1.SeriesCount > 0 then
    Chart1.FreeAllSeries();
  for i := 0 to Chart1.CustomAxes.Count - 1 do
    Chart1.CustomAxes.Items[i].Grid.Visible := false;

  for i := 0 to 2 do
    begin
      with Chart1.AddSeries(TLineSeries) as TLineSeries do
        begin
          FillSampleValues;
          CustomHorizAxis := Chart1.CustomAxes.Items[0];
          CustomVertAxis := Chart1.CustomAxes.Items[1];
          LinePen.Visible := true;
          LinePen.Style := psSolid;
          LinePen.Color := Color;
          LinePen.Width := 2;
          Pointer.Visible := false;
        end;
    end;

  ChartTool1.Series := Chart1[0];
  RadioGroup1.ItemIndex := 2;
  RadioGroup1Click(Sender);
end;

function TForm1.InterpolateLineSeriesVertical(Series: TChartSeries; XValue: Double): Double;
begin
  result := InterpolateLineSeriesVertical(Series, Series.FirstDisplayedIndex, Series.LastValueIndex, XValue);
end;

function TForm1.InterpolateLineSeriesVertical(Series: TChartSeries; FirstIndex, LastIndex: Integer; XValue: Double): Double;
var
  Index: Integer;
  dx, dy: Double;
begin
  if (Series.Count > 0) and (LastIndex > FirstIndex) then
    begin
      // for Index := FirstIndex + 1 to LastIndex do
      for Index := LastIndex downto FirstIndex + 1 do
        if ((XValue >= Series.XValues.Value[Index - 1]) and (XValue < Series.XValues.Value[Index])) or
          ((XValue <= Series.XValues.Value[Index - 1]) and (XValue > Series.XValues.Value[Index])) then
          break;
      // safeguard
      if (Index < 1) then
        Index := 1
      else if (Index >= Series.Count) then
        Index := Series.Count - 1;
      // y=(y2-y1)/(x2-x1)*(x-x1)+y1
      dx := Series.XValues.Value[Index] - Series.XValues.Value[Index - 1];
      dy := Series.YValues.Value[Index] - Series.YValues.Value[Index - 1];
      if (dx <> 0) then
        result := dy * (XValue - Series.XValues.Value[Index - 1]) / dx + Series.YValues.Value[Index - 1]
      else
        result := 0;
    end
  else
    result := 0;
end;

procedure TForm1.RadioGroup1Click(Sender: TObject);
begin
  if RadioGroup1.ItemIndex < 0 then
    RadioGroup1.ItemIndex := 0;
  if RadioGroup1.ItemIndex = 0 then
    ChartTool1.Active := false
  else
    begin
      ChartTool1.Active := true;
      if RadioGroup1.ItemIndex = 1 then
        ChartTool1.Style := cssHorizontal { cssHorizontal, cssVertical }
      else if RadioGroup1.ItemIndex = 2 then
        ChartTool1.Style := cssVertical; { cssHorizontal, cssVertical }
    end;
end;

function TForm1.InterpolateLineSeriesHorizontal(Series: TChartSeries; YValue: Double): Double;
begin
  result := InterpolateLineSeriesHorizontal(Series, Series.FirstDisplayedIndex, Series.LastValueIndex, YValue);
end;

function TForm1.InterpolateLineSeriesHorizontal(Series: TChartSeries; FirstIndex, LastIndex: Integer; YValue: Double): Double;
var
  Index: Integer;
  dx, dy: Double;
begin
  if (Series.Count > 0) and (LastIndex > FirstIndex) then
    begin
      // for Index := FirstIndex + 1 to LastIndex do
      for Index := LastIndex downto FirstIndex + 1 do
        begin
          if ((YValue >= Series.YValues.Value[Index - 1]) and (YValue < Series.YValues.Value[Index])) or
            ((YValue <= Series.YValues.Value[Index - 1]) and (YValue > Series.YValues.Value[Index])) then
            break;
        end;
      // safeguard
      if (Index < 1) then
        Index := 1
      else if (Index >= Series.Count) then
        Index := Series.Count - 1;
      // x=(x2-x1)/(y2-y1)*(y-y1)+x1
      dx := Series.XValues.Value[Index] - Series.XValues.Value[Index - 1];
      dy := Series.YValues.Value[Index] - Series.YValues.Value[Index - 1];
      if (dy <> 0) then
        result := dx * (YValue - Series.YValues.Value[Index - 1]) / dy + Series.XValues.Value[Index - 1]
      else
        result := 0;
    end
  else
    result := 0;
end;

procedure TForm1.Chart1AfterDraw(Sender: TObject);
var
  xs, ys, i: Integer;
begin
  inherited;
  if Chart1.SeriesCount > 0 then
    begin
      if RadioGroup1.ItemIndex = 1 then
        begin
          ys := Chart1.CustomAxes.Items[1].CalcYPosValue(tmpYVal);
          for i := 0 to Chart1.SeriesCount - 1 do
            begin
              xs := Chart1[i].GetHorizAxis.CalcXPosValue(InterpolateLineSeriesHorizontal(Chart1[i], tmpYVal));
              Chart1.Canvas.Brush.Color := Chart1[i].Color;
              Chart1.Canvas.Ellipse(xs - 4, ys - 4, xs + 4, ys + 4);
            end;
        end
      else if RadioGroup1.ItemIndex = 2 then
        begin
          xs := Chart1.CustomAxes.Items[0].CalcXPosValue(tmpXVal);
          for i := 0 to Chart1.SeriesCount - 1 do
            begin
              ys := Chart1[i].GetVertAxis.CalcYPosValue(InterpolateLineSeriesVertical(Chart1[i], tmpXVal));
              Chart1.Canvas.Brush.Color := Chart1[i].Color;
              Chart1.Canvas.Ellipse(xs - 4, ys - 4, xs + 4, ys + 4);
            end;
        end;
    end;
end;

procedure TForm1.ChartTool1Change(Sender: TCursorTool; x, y: Integer; const XValue, YValue: Double; Series: TChartSeries; ValueIndex: Integer);
var
  i: Integer;
  S: string;
begin
  if (Chart1.SeriesCount > 0) and ((RadioGroup1.ItemIndex = 1) or (RadioGroup1.ItemIndex = 2)) then
    begin
      if RadioGroup1.ItemIndex = 1 then
        begin
          tmpYVal := YValue;
          S := Chart1.CustomAxes.Items[1].Title.Caption;
          S := '[' + S + ' = ' + FloatToStrF(YValue, ffNumber, 8, 2) + ']' + #13#10;
          for i := 0 to Chart1.SeriesCount - 1 do
            S := S + Chart1[i].Title + ': ' + FloatToStrF(InterpolateLineSeriesHorizontal(Chart1[i], YValue), ffNumber, 8, 2) + #13#10;
        end
      else if RadioGroup1.ItemIndex = 2 then
        begin
          tmpXVal := XValue;
          S := Chart1.CustomAxes.Items[0].Title.Caption;
          S := '[' + S + ' = ' + FloatToStrF(XValue, ffNumber, 8, 2) + ']' + #13#10;
          for i := 0 to Chart1.SeriesCount - 1 do
            S := S + Chart1[i].Title + ': ' + FloatToStrF(InterpolateLineSeriesVertical(Chart1[i], XValue), ffNumber, 8, 2) + #13#10;
        end;
       ChartTool1.AxisAnnotation.Text := S;  //????
    end;
end;

procedure TForm1.ButtonExitClick(Sender: TObject);
begin
  close;
end;

end.
